#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class HardwareObjectsController < ApplicationController
  before_action :set_hardware_object, only: [:edit, :update, :destroy]

  layout false, only: [:new, :edit, :import]

  # GET /hardware_objects
  # GET /hardware_objects.json
  def index
    session.delete(:search)
    @hardware_objects = HardwareObject.all
                          .order(created_at: :desc)
                          .paginate(page: 1, per_page: current_user.per_page_count)
  end

  # GET /hardware_objects/new
  def new
    @hardware_object = HardwareObject.new(product: params[:product])
  end

  # POST /hardware_objects
  # POST /hardware_objects.json
  def create
    @hardware_object = HardwareObject.new(hardware_object_params)

    if @hardware_object.save
      if session[:search].present?
        filter = ['( LOWER(product) LIKE :search or ' \
                'LOWER(family) LIKE :search or LOWER(vendor) LIKE :search)',
                search: "%#{session[:search].try(:downcase)}%"]
        @hardware_objects = HardwareObject.where(filter)
      else
        @hardware_objects = HardwareObject.all
      end
      new_page = params[:page] ? params[:page].to_i : 1
      @hardware_objects = @hardware_objects.order(created_at: :desc)
                            .paginate(page: new_page, per_page: current_user.per_page_count)
      flash.now[:notice] = I18n.t('hardware_objects.create.success')
    else
      flash[:alert] = I18n.t('hardware_objects.create.error')
    end
  end

  # PATCH/PUT /hardware_objects/1
  # PATCH/PUT /hardware_objects/1.json
  def update
    if @hardware_object.update(hardware_object_params)
      flash.now[:notice] = I18n.t('hardware_objects.update.success')
    else
      flash[:alert] = I18n.t('hardware_objects.update.error')
    end
  end

  # DELETE /hardware_objects/1
  # DELETE /hardware_objects/1.json
  def destroy
    if @hardware_object && @hardware_object.destroy
      # Keep filter when deleting
      if session[:search].present?
        filter = ['( LOWER(product) LIKE :search or ' \
                'LOWER(family) LIKE :search or LOWER(vendor) LIKE :search)',
                search: "%#{session[:search].try(:downcase)}%"]
        @hardware_objects = HardwareObject.where(filter)
      else
        @hardware_objects = HardwareObject.all
      end

      # Keep page in range and deleting last page if it is empty
      current_page = [params[:page].to_i, 1].max
      max_page = (@hardware_objects.any? ? (@hardware_objects.count.to_f / current_user.per_page_count).ceil : 1)
      new_page = [current_page, max_page].min

      @hardware_objects = @hardware_objects
                            .order(created_at: :desc)
                            .paginate(page: new_page, per_page: current_user.per_page_count)
      flash.now[:notice] = I18n.t('hardware_objects.destroy.success')
    else
      flash[:alert] = I18n.t('hardware_objects.destroy.error')
    end
  end

  #
  # Search in applications
  #
  def search
    if params[:search].blank?
      session.delete(:search)
      @hardware_objects = HardwareObject.all
                          .order(created_at: :desc)
                          .paginate(page: params[:page], per_page: current_user.per_page_count)
    else
      session[:search] = params[:search] if params[:search].present?
      filter = ['( LOWER(product) LIKE :search or ' \
              'LOWER(family) LIKE :search or LOWER(vendor) LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]
      @hardware_objects = HardwareObject.where(filter)
                            .order(created_at: :desc)
                            .paginate(page: params[:page], per_page: current_user.per_page_count)
    end   
  end

  #
  # Export all applications as CSV
  #
  def export
    filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') + '-Applications.csv' }
    filter = ['( LOWER(product) LIKE :search or ' \
              'LOWER(family) LIKE :search or LOWER(vendor) LIKE :search)',
              search: "%#{params[:search].try(:downcase)}%"]
    content = HardwareObject.where(filter).order(:name)
              .map { |o| "#{o.product};#{o.family};#{o.vendor}" }.join("\n")
    send_data content, filename: filename
  end

  #
  # Import applications from a given CSV file.
  #
  def import_file
    content_file = params[:hardware_object][:csv_file]

    if params[:add_vars] == 'true'
      count = HardwareObject.add_apps_from_file(content_file)
      msg = "#{count[0]} #{I18n.t('controllers.added')} #{count[1]} #{I18n.t('controllers.updated')}"
    else
      count = HardwareObject.replace_apps_from_file(content_file)
      msg = "#{count} #{I18n.t('controllers.added')}"
    end

    redirect_to objects_path, notice: msg
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_hardware_object
    @hardware_object = HardwareObject.find_by(id: params[:id])
    return if @hardware_object
    # The object doesn't exist!
    rb_flash "#{I18n.t('controllers.not_exist')}", type: :error
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def hardware_object_params
    params.require(:hardware_object).permit(:product, :family, :vendor, :page, :search)
  end
end
